# !/sbin/sh

######################
## Gfiles by drakaz ##
######################

# Backup/restore Google Apps on Galaxo/Galaxhero

APPS="Gmail.apk GmailProvider.apk GoogleApps.apk GooglePartnerSetup.apk GoogleSettingsProvider.apk gtalkservice.apk  Maps.apk MediaUploader.apk SetupWizard.apk Street.apk Talk.apk Vending.apk YouTube.apk"
FRAMEWORK="com.google.android.gtalkservice.jar com.google.android.gtalkservice.odex com.google.android.maps.jar com.google.android.maps.odex"
PERMS="com.google.android.gtalkservice.xml com.google.android.maps.xml"
LIBS="libaudioeq.so libcamera.so libgps.so libhgl.so libloc_api.so libOmxCore.so libOmxH264Dec.so libOmxMpeg4Dec.so libOmxVidEnc.so libopencorehw.so libmm-adspsvc.so"

TMPDIR="/sdcard/TMPGALAXO/"
UPDATEFILE="/sdcard/II5.tar"
CHECKSUM="45fe8c2319e8f8d1a74d5881d70bed18"


if [ $1 = "oneinall" ]
then

	# Verification du fichier de maj
	if [ ! -s $UPDATEFILE ]
	then
		echo "Erreurs : le fichier de mise à jour est absent. Veuillez déposer le fichier de mise à jour dans /sdcard/ renommé en $2 (/sdcard/$2)"
		exit 1
	else
		echo "Fichier mise à jour trouve : $UPDATEFILE"
	fi

	# On remount en rw
	/tmp/RECTOOLS/remountrw

	# Creation du rep temporaire
	mkdir $TMPDIR

	if [ ! -d $TMPDIR ]
	then
		echo "Erreur lors de la création du répértoire temporaire"
		exit 1	
	fi      

	# Extraction de l'image system
	echo -n "Extraction de l'image system"
	tar -xvf $UPDATEFILE system -C $TMPDIR
	echo " "

	# Verification du fichier image
	if [ ! -s $TMPDIR/system ]
	then
		echo "Erreur lors de l'extraction du fichier image"
		rm -rf $TMPDIR
		exit 1
	fi

	# Verification du MD5
	echo -n "Verification du checksum"
	if [ ! $CHECKSUM = `md5sum $TMPDIR/system | awk -F " " '{print $1}'` ]
	then
		echo "Erreur de checksum, veuillez verifier la version de votre update"
		rm -rf $TMPDIR
		exit 1
	else
		echo "Checksum OK"
	fi	

	# Extraction du fichier image
	echo -n "Extraction du fichier image"
	cd $TMPDIR; unyaffs system


	# On vérifit l'extraction
	for i in `echo $APPS`
	do
		if [ ! -s /$TMPDIR/app/$i ]
		then
			echo "Erreurs d'extraction du fichier image : fichier /system/app/$i absent"
			rm -rf $TMPDIR
			exit 1
		fi
	done
	for i in `echo $FRAMEWORK`
	do
		if [ ! -s /$TMPDIR/framework/$i ]
		then
			echo "Erreurs d'extraction du fichier image : fichier /system/framework/$i absent"
			rm -rf $TMPDIR
			exit 1
		fi
	done
	for i in `echo $PERMS`
	do
		if [ ! -s /$TMPDIR/etc/permissions/$i ]
		then
			echo "Erreurs d'extraction du fichier image : fichier /system/etc/permissions/$i absent"
			rm -rf $TMPDIR
			exit 1
		fi
	done
	for i in `echo $LIBS`
	do
		if [ ! -s /$TMPDIR/lib/$i ]
		then
			echo "Erreurs d'extraction du fichier image : fichier /system/lib/$i absent"
			rm -rf $TMPDIR
			exit 1
		fi
	done



	# On restore les applications
	echo -n "Restauration des applications"
	for i in `echo $APPS`
	do	
		cp /$TMPDIR/app/$i /system/app/$i
	done
	echo " "
	
	# On restore la framework
	echo -n "Restauration de la framework"
	for i in `echo $FRAMEWORK`
	do	
		cp /$TMPDIR/framework/$i  /system/framework/$i
	done
	echo " "
	
	# On restore les permissions
	echo -n "Restauration des permissions"
	for i in `echo $PERMS`
	do	
		cp /$TMPDIR/etc/permissions/$i  /system/etc/permissions/$i
	done
	echo " "

	# On restore les libs
	echo -n "Restauration des librairies"
	for i in `echo $LIBS`
	do	
		cp /$TMPDIR/lib/$i  /system/lib/$i
	done	
	echo " "

	# On vérifit le restore
	for i in `echo $APPS`
	do
		if [ ! -s /system/app/$i ]
		then
			echo "Erreur de restauration : fichier /system/app/$i absent"
			rm -rf $TMPDIR
			exit 1
		fi
	done
	for i in `echo $FRAMEWORK`
	do
		if [ ! -s /system/framework/$i ]
		then
			echo "Erreur de restauration : fichier /system/framework/$i absent"
			rm -rf $TMPDIR
			exit 1
		fi
	done
	for i in `echo $PERMS`
	do
		if [ ! -s /system/etc/permissions/$i ]
		then
			echo "Erreur de restauration : fichier /system/etc/permissions/$i absent"
			rm -rf $TMPDIR
			exit 1
		fi
	done
	for i in `echo $LIBS`
	do
		if [ ! -s /system/lib/$i ]
		then
			echo "Erreur de restauration : fichier /system/lib/$i absent"
			rm -rf $TMPDIR
			exit 1
		fi
	done
	echo "Restore successfull"
	rm -rf $TMPDIR
	sync
	/tmp/RECTOOLS/remountro
	exit 0
fi






if [ $1 = "backup" ]
then

# On remount en rw
	/tmp/RECTOOLS/remountrw

# On Purge l'ancien backup
	rm -rf /sdcard/Gfiles/

# On vérifit la présence des repertoires de sauvegardes et on créer si besoin
	if [ ! -d /sdcard/Gfiles ]
	then
		mkdir /sdcard/Gfiles	
	fi      
	if [ ! -d /sdcard/Gfiles/apps ]
	then
		mkdir /sdcard/Gfiles/apps	
	fi    
	if [ ! -d /sdcard/Gfiles/lib ]
	then
		mkdir /sdcard/Gfiles/lib	
	fi      
	if [ ! -d /sdcard/Gfiles/etc ]
	then
		mkdir /sdcard/Gfiles/etc	
	fi          
	if [ ! -d /sdcard/Gfiles/etc/permissions ]
	then
		mkdir /sdcard/Gfiles/etc/permissions	
	fi 
	if [ ! -d /sdcard/Gfiles/framework ]
	then
		mkdir /sdcard/Gfiles/framework	
	fi        

# On sauvegardes les applications
	for i in `echo $APPS`
	do	
		cp /system/app/$i /sdcard/Gfiles/apps/$i
	done
	
# On sauvegarde la framework
	for i in `echo $FRAMEWORK`
	do	
		cp /system/framework/$i /sdcard/Gfiles/framework/$i
	done

# On Sauvegarde les libs
	for i in `echo $LIBS`
	do	
		cp /system/lib/$i /sdcard/Gfiles/lib/$i
	done

	
# On sauvegardes les permissions
	for i in `echo $PERMS`
	do	
		cp /system/etc/permissions/$i /sdcard/Gfiles/etc/permissions/$i
	done

# On vérifit le backup
	for i in `echo $APPS`
	do
		if [ ! -s /sdcard/Gfiles/apps/$i ]
		then
			echo "Erreurs : fichier /sdcard/Gfiles/apps/$i absent"
			exit 1
		fi
	done
	for i in `echo $FRAMEWORK`
	do
		if [ ! -s /sdcard/Gfiles/framework/$i ]
		then
			echo "Erreurs : fichier /sdcard/Gfiles/framework/$i absent"
			exit 1
		fi
	done
	for i in `echo $PERMS`
	do
		if [ ! -s /sdcard/Gfiles/etc/permissions/$i ]
		then
			echo "Erreurs : fichier /sdcard/Gfiles/etc/permissions/$i absent"
			exit 1
		fi
	done

	for i in `echo $LIBS`
	do
		if [ ! -s /sdcard/Gfiles/lib/$i ]
		then
			echo "Erreurs : fichier /sdcard/Gfiles/lib/$i absent"
			exit 1
		fi
	done

	/tmp/RECTOOLS/remountro
	sync
	echo "Backup successfull"
	reboot recovery
	exit 0
fi

if [ $1 = "restore" ]
then

# On remount en rw
	/tmp/RECTOOLS/remountrw

# On vérifit le backup
	for i in `echo $APPS`
	do
		if [ ! -s /sdcard/Gfiles/apps/$i ]
		then
			echo "Erreurs : fichier /sdcard/Gfiles/apps/$i absent"
			exit 1
		fi
	done
	for i in `echo $FRAMEWORK`
	do
		if [ ! -s /sdcard/Gfiles/framework/$i ]
		then
			echo "Erreurs : fichier /sdcard/Gfiles/framework/$i absent"
			exit 1
		fi
	done
	for i in `echo $PERMS`
	do
		if [ ! -s /sdcard/Gfiles/etc/permissions/$i ]
		then
			echo "Erreurs : fichier /sdcard/Gfiles/etc/permissions/$i absent"
			exit 1
		fi
	done

	for i in `echo $LIBS`
	do
		if [ ! -s /sdcard/Gfiles/lib/$i ]
		then
			echo "Erreurs : fichier /sdcard/Gfiles/lib/$i absent"
			exit 1
		fi
	done


# On restore les applications
	for i in `echo $APPS`
	do	
		cp /sdcard/Gfiles/apps/$i /system/app/$i
	done
	
# On restore la framework
	for i in `echo $FRAMEWORK`
	do	
		cp /sdcard/Gfiles/framework/$i  /system/framework/$i
	done
	
# On restore les permissions
	for i in `echo $PERMS`
	do	
		cp /sdcard/Gfiles/etc/permissions/$i  /system/etc/permissions/$i
	done

# On restore les libs
	for i in `echo $LIBS`
	do	
		cp /sdcard/Gfiles/lib/$i  /system/lib/$i
	done

# On vérifit le restore
	for i in `echo $APPS`
	do
		if [ ! -s /system/app/$i ]
		then
			echo "Erreurs : fichier /system/app/$i absent"
			exit 1
		fi
	done
	for i in `echo $FRAMEWORK`
	do
		if [ ! -s /system/framework/$i ]
		then
			echo "Erreurs : fichier /system/framework/$i absent"
			exit 1
		fi
	done
	for i in `echo $PERMS`
	do
		if [ ! -s /system/etc/permissions/$i ]
		then
			echo "Erreurs : fichier /system/etc/permissions/$i absent"
			exit 1
		fi
	done
	for i in `echo $LIBS`
	do
		if [ ! -s /system/lib/$i ]
		then
			echo "Erreurs : fichier /system/lib/$i absent"
			exit 1
		fi
	done

	echo "Restore successfull"
	sync
	/tmp/RECTOOLS/remountro
	exit 0
fi



